//-*-c++-*------------------------------------------------------------
//
// File name : bioArithExp.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Tue May  5 14:14:41 2009
//
//--------------------------------------------------------------------


#include "patMath.h"
#include "patDisplay.h"
#include "patErrNullPointer.h"
#include "bioArithExp.h"
#include "bioArithMult.h"
#include "bioArithCompositeLiteral.h"
#include "bioArithNamedExpression.h"
#include "bioArithConstant.h"
#include "bioLiteralRepository.h"
#include "bioExpressionRepository.h"

bioArithExp::bioArithExp(bioExpressionRepository* rep,
			 patULong par,
                         patULong left,
			 patError*& err) 
  : bioArithUnaryExpression(rep, par,left,err) {


}

bioArithExp::~bioArithExp() {}

patString bioArithExp::getOperatorName() const {
  return ("exp") ;
}

patReal bioArithExp::getValue(patBoolean prepareGradient, 
			      patError*& err) {
#ifdef BIO_CHECK
  if (child == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patReal();
  }
#endif
  if (child->isStructurallyZero()) {
    return 1.0 ;
  }
  else {
    patReal childValue = child->getValue(prepareGradient,err) ;
#ifdef BIO_CHECK
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patReal();
    }
#endif    
    patReal result = patExp(childValue) ;
    return result ;
  }
}


bioExpression* bioArithExp::getDerivative(patULong aLiteralId, patError*& err) const {
  if (child == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return NULL;
  }
  
  bioExpression* value = child->getDeepCopy(theRepository,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }
  bioExpression* deriv = child->getDerivative(aLiteralId,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }

  bioExpression* e = new bioArithExp(theRepository, patBadId, value->getId(),err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }
  bioExpression* result = new bioArithMult(theRepository,patBadId,deriv->getId(),e->getId(),err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
  return result ;
}


bioArithExp* bioArithExp::getDeepCopy(bioExpressionRepository* rep,
				      patError*& err) const {
  bioExpression* leftClone(NULL) ;
  //bioExpression* rightClone(NULL) ;
  if (child != NULL) {
    leftClone = child->getDeepCopy(rep,err) ;
  }
  //if (rightChild != NULL) {
  //  rightClone = rightChild->getDeepCopy(err) ;
  //}

  bioArithExp* theNode = 
    new bioArithExp(rep,patBadId,leftClone->getId(),err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }
  return theNode ;
}

bioArithExp* bioArithExp::getShallowCopy(bioExpressionRepository* rep,
				      patError*& err) const {

  bioArithExp* theNode = 
    new bioArithExp(rep,patBadId,child->getId(),err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }
  return theNode ;
}


patString bioArithExp::getCppCode(patString prefix, patError*& err) {
  if (codeMustBeGenerated) {
  if (child == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  
  patString vLeft = child->getCppCode(prefix, err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }


  stringstream str ;
  str << vLeft << endl ;
  str << getCppResultDeclaration(prefix) << " = patExp(" << child->getCppResult(prefix) << ");" << endl ;
  //  str << "DEBUG_MESSAGE(\"" << getCppResult(prefix) << "= \" << " << getCppResult(prefix) << ");" << endl ;
  codeMustBeGenerated = patFALSE ;

  
  return patString(str.str()) ;
  }
  else {
    return patString() ;
  }
}

patString bioArithExp::getSimpleCppCode(patError*& err) {
  if (child == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  patString vLeft = child->getSimpleCppCode(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }
  stringstream str ;
  str << "patExp(" << vLeft << ")" ;
  return patString(str.str()) ;
}

patString bioArithExp::getExpressionString() const {
  stringstream str ;
  str << "$E" ;
  if (child != NULL) {
    str << '{' << child->getExpressionString() << '}' ;
  }
  return patString(str.str()) ;
}



bioFunctionAndDerivatives* bioArithExp::getNumericalFunctionAndGradient(vector<patULong> literalIds, patError*& err) {
  if (result.empty()) {
    result.resize(literalIds.size()) ;
  }
  bioFunctionAndDerivatives* c = child->getNumericalFunctionAndGradient(literalIds,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }

  result.theFunction =  patExp((c->theFunction)) ;
  for (patULong i = 0 ; i < literalIds.size() ; ++i) {
    if (c->theGradient[i] == 0.0) {
      result.theGradient[i] = 0.0 ;

    }
    else {
      result.theGradient[i] = result.theFunction * c->theGradient[i] ;
    }
  }
  return &result ;
  
}
