//-*-c++-*------------------------------------------------------------
//
// File name : bioMyObjects.h
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Sat Dec 19 19:43:51 2009
//
//--------------------------------------------------------------------

// We declare here the user defined objects

#ifndef bioMyObjects_h
#define bioMyObjects_h

#include "bioPythonWrapper.h"
#include "bioStatistics.h"
#include "bioConstraints.h"
#include "bioIteratorSpan.h"

class bioExpression ;
class bioArithListOfExpressions ;

typedef struct{
  trVector* grad;
  trHessian* hessian ;
  trHessian* bhhh ;
  patBoolean* success ;
  patError* err ;
  bioSample* sample ;
  patReal result ;
  bioIteratorSpan subsample ; 
  bioExpression* theExpression ;
  bioArithListOfExpressions* theFunctionAndDerivatives ;
} bioThreadArg ;

class myBiogeme: public bioPythonWrapper {

public :

  myBiogeme(bioExpression* expr) ;
  ~myBiogeme() ;
  patReal getFunction(trVector* x,
		      patBoolean* success,
		      patError*& err) ;
  trVector* getGradient(trVector* x,
			trVector* grad,
			patBoolean* success,
			patError*& err)  ;
  
  patReal getFunctionAndGradient(trVector* x,
				 trVector* grad,
				 patBoolean* success,
				 patError*& err); 

  trHessian* computeHessian(patVariables* x,
			    trHessian& hessian,
			    patBoolean* success,
			    patError*& err) ; 
  
  trHessian* getCheapHessian(trVector* x,
			     trHessian& hessian,
			     patBoolean* success,
			     patError*& err) ;

   patBoolean isCheapHessianAvailable() ;

  trVector getHessianTimesVector(trVector* x,
				 const trVector* v,
				 patBoolean* success,
				 patError*& err)  ;
  
  patBoolean isGradientAvailable(patError*& err) const ;

  patBoolean isHessianAvailable(patError*& err)  const ;

  patBoolean isHessianTimesVectorAvailable(patError*& err) const ;

  unsigned long getDimension(patError*& err) const ;
  
  trVector getCurrentVariables(patError*& err) const ;
  
  patBoolean isUserBased() const ;

  void generateCppCode(patError*& err) ;


  void generateDerivatives(patError*& err) ;
 private:
  trHessian bhhh ;
  trHessian hessian ;
  vector<trVector> threadGrad ;
  vector<trHessian> threadBhhh ;
  vector<trHessian> threadHessian ;

  bioExpression* theExpression ;
  bioArithListOfExpressions* theFunctionAndDerivatives ;
};

class myBioStatistics: public bioStatistics {
public:
  
  void bio__generateStatistics(bioSample* sample, patError*& err) ;
};

class myBioConstraints: public bioConstraints {
public:
  
  void addConstraints() ;
};



#endif
