//-*-c++-*------------------------------------------------------------
//
// File name : bioParameters.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Fri Jul 17 14:17:08 2009
//
//--------------------------------------------------------------------

#include "patDisplay.h"
#include "patErrMiscError.h"
#include "patErrNullPointer.h"
#include "bioParameters.h"

bioParameters* bioParameters::the() {
  static bioParameters* singleInstance = NULL ;

  if (singleInstance==NULL) {
    singleInstance = new bioParameters ;
  }
  return singleInstance ;
}

bioParameters::bioParameters() {
  stringValues["HeaderOfRowId"] = patString("__rowId__") ;
  stringValues["RandomDistribution"] = patString("PSEUDO") ;
  stringValues["warningSign"] = patString("*") ;
  stringValues["optimizationAlgorithm"] = patString("BIO") ;
  stringValues["stopFileName"] = patString("STOP") ;

  integerValues["NbrOfDraws"] = 1000 ;
  integerValues["Seed"] = 9021967 ;
  integerValues["printPValue"] = 1 ;
  integerValues["computeInitLoglikelihood"] = 1 ;
  integerValues["checkDerivatives"] = 0 ;
  integerValues["simplifyExpressions"] = 1 ;
  integerValues["buildAnalyticalGradient"] = 1 ;
  integerValues["varCovarFromBHHH"] = 0 ;
  integerValues["numberOfThreads"] = 2 ;
  integerValues["dumpDrawsOnFile"] = 0 ;
  integerValues["antitheticDraws"] = 1 ;
  integerValues["warnsForIllegalElements"] = 1 ;
  integerValues["cfsqpMode"] = 100 ;
  integerValues["cfsqpIprint"] = 2 ;
  integerValues["cfsqpMaxIter"] = 1000 ;
  integerValues["svdMaxIter"] = 60 ;
  integerValues["maxPrimeNumbers"] = 1000 ;
  integerValues["firstIdOfLiterals"] = 10000 ;

  integerValues["BTRMaxIter"] = 1000 ;
  integerValues["BTRInitQuasiNewtonWithTrueHessian"] = 0 ;
  integerValues["BTRSignificantDigits"] = 7 ;
  integerValues["BTRExactHessian"] = 0 ;
  integerValues["BTRCheapHessian"] = 1 ;
  integerValues["BTRStartDraws"] = 10 ;
  integerValues["BTRIncreaseDraws"] = 2 ;
  integerValues["maxGcpIter"] = 10000 ;
  integerValues["BTRUnfeasibleCGIterations"] = 0 ;
  integerValues["BTRQuasiNewtonUpdate"] = 1 ;
  integerValues["BTRUsePreconditioner"] = 1 ;

  integerValues["solvoptMaxIter"] = 1000 ;
  integerValues["solvoptDisplay"] = 1 ;
  integerValues["donlp2nreset"] = 9 ;

  realValues["cfsqpEps"] = 6.05545e-06 ;
  realValues["cfsqpEpsEqn"] = 6.05545e-06 ;
  realValues["cfsqpUdelta"] = 0.0 ;
  realValues["BTRInitRadius"] = 1.0 ;
  realValues["tTestThreshold"] = 1.96 ;
  realValues["singularValueThreshold"] = 1.0e-4 ;
  realValues["missingValue"] = 99999.0 ;
  
  realValues["BTREta1"] = 0.01 ;
  realValues["BTREta2"] = 0.9 ;
  realValues["BTRGamma1"] = 0.5 ;
  realValues["BTRGamma2"] = 0.5 ;
  realValues["BTRIncreaseTRRadius"] = 2 ; 
  realValues["BTRMaxTRRadius"] = 1.0e10 ;
  realValues["BTRTypf"] = 1.0 ;
  realValues["BTRTolerance"] = 6.05545e-06 ;
  realValues["TolSchnabelEskow"] = 0.00492157 ;
  realValues["BTRInitRadius"] = 1.0 ;
  realValues["BTRMinTRRadius"] = 1.0e-7 ;
  realValues["BTRArmijoBeta1"] = 0.1 ;
  realValues["BTRArmijoBeta2"] = 0.9 ;
  realValues["TSFractionGradientRequired"] = 0.1 ;
  realValues["TSExpTheta"] = 0.5 ;
  realValues["BTRKappaUbs"] = 0.1 ;
  realValues["BTRKappaLbs"] = 0.9 ;
  realValues["BTRKappaFrd"] = 0.5 ;
  realValues["BTRKappaEpp"] = 0.25 ;
  realValues["solvoptErrorArgument"] = 1.0e-4 ;
  realValues["solvoptErrorFunction"] = 1.0e-6 ;
  realValues["donlp2epsx"] = 1.0e-5 ;
  realValues["donlp2delmin"] = 1.0e-6 ;
  realValues["donlp2smallw"] = 3.66685e-11 ;
  realValues["donlp2epsdif"] = 0.0 ;
}


patReal bioParameters::getValueReal(patString p, patError*& err) const {
  map<patString,patReal>::const_iterator found = realValues.find(p) ;
  if (found == realValues.end()) {
    stringstream str ;
    str << "Unknown parameter: " << p ;
    err = new patErrMiscError(str.str()) ;
    WARNING(err->describe()) ;
    return patReal() ;
  }
  else {
    return found->second ;
  }
}


patReal bioParameters::getValueReal(patString p) const {
  map<patString,patReal>::const_iterator found = realValues.find(p) ;
  if (found == realValues.end()) {
    return patReal() ;
  }
  else {
    return found->second ;
  }
}

patString bioParameters::getValueString(patString p, patError*& err) const {
  map<patString,patString>::const_iterator found = stringValues.find(p) ;
  if (found == stringValues.end()) {
    stringstream str ;
    str << "Unknown parameter: " << p ;
    err = new patErrMiscError(str.str()) ;
    WARNING(err->describe()) ;
    return patString() ;
  }
  else {
    return found->second ;
  }

}

patString bioParameters::getValueString(patString p) const {
  map<patString,patString>::const_iterator found = stringValues.find(p) ;
  if (found == stringValues.end()) {
    return patString() ;
  }
  else {
    return found->second ;
  }

}

long bioParameters::getValueInt(patString p, patError*& err) const {
  map<patString,long>::const_iterator found = integerValues.find(p) ;
  if (found == integerValues.end()) {
    stringstream str ;
    str << "Unknown parameter: " << p ;
    err = new patErrMiscError(str.str()) ;
    WARNING(err->describe()) ;
    return long() ;
  }
  else {
    return found->second ;
  }
}

long bioParameters::getValueInt(patString p) const {
  map<patString,long>::const_iterator found = integerValues.find(p) ;
  if (found == integerValues.end()) {
    return long() ;
  }
  else {
    return found->second ;
  }
}

void bioParameters::setValueReal(patString p, patReal v, patError*& err) {
  if (isStringParam(p)) {
    stringstream str ;
    str << "Paramater " << p << " has already been defined as a string parameter with value " << stringValues[p] << " and cannot become a real parameter with value " << v ;
    err = new patErrMiscError(str.str()) ;
    WARNING(err->describe()) ;
    return ;
  }
  if (isIntParam(p)) {
    stringstream str ;
    str << "Paramater " << p << " has already been defined as an integer parameter with value " << integerValues[p] << " and cannot become a real parameter with value " << v ;
    err = new patErrMiscError(str.str()) ;
    WARNING(err->describe()) ;
    return ;
  }
  realValues[p] = v ;
}

void bioParameters::setValueString(patString p, patString v, patError*& err) {
  if (isIntParam(p)) {
    stringstream str ;
    str << "Paramater " << p << " has already been defined as an integer parameter with value " << integerValues[p] << " and cannot become a string parameter with value " << v ;
    err = new patErrMiscError(str.str()) ;
    WARNING(err->describe()) ;
    return ;
  }
  if (isRealParam(p)) {
    stringstream str ;
    str << "Paramater " << p << " has already been defined as a real parameter with value " << integerValues[p] << " and cannot become a string parameter with value " << v ;
    err = new patErrMiscError(str.str()) ;
    WARNING(err->describe()) ;
    return ;
    
  }
  stringValues[p] = v ;
}

void bioParameters::setValueInt(patString p, long v, patError*& err) {
  if (isStringParam(p)) {
    stringstream str ;
    str << "Paramater " << p << " has already been defined as a string parameter with value " << stringValues[p] << " and cannot become an integer parameter with value " << v ;
    err = new patErrMiscError(str.str()) ;
    WARNING(err->describe()) ;
    return ;
  }
  if (isRealParam(p)) {
    stringstream str ;
    str << "Paramater " << p << " has already been defined as a real parameter with value " << integerValues[p] << " and cannot become a integer parameter with value " << v ;
    err = new patErrMiscError(str.str()) ;
    WARNING(err->describe()) ;
    return ;
    
  }
  integerValues[p] = v ;
}

void bioParameters::readParamFile(patString aFile, patError*& err) {

  err = new patErrMiscError("Not yet implemented") ;
  WARNING(err->describe()) ;
  return ;
  
}


patBoolean bioParameters::isRealParam(patString paramName) const {
  map<patString,patReal>::const_iterator found ;
  found = realValues.find(paramName) ;
  return (found != realValues.end()) ;
}
patBoolean bioParameters::isStringParam(patString paramName) const {
  map<patString,patString>::const_iterator found ;
  found = stringValues.find(paramName) ;
  return (found != stringValues.end()) ;
  
}
patBoolean bioParameters::isIntParam(patString paramName) const {
  map<patString,long int>::const_iterator found ;
  found = integerValues.find(paramName) ;
  return (found != integerValues.end()) ;
}

void bioParameters::setParameters(map<patString, patString>* aDict, 
				  patError*& err) {
  if (aDict == NULL) {
    err = new patErrNullPointer("map<patString, patString>") ;
    WARNING(err->describe()) ;
    return ;
  }
  vector<patString> unknown ;
  for (map<patString, patString>::iterator i = aDict->begin() ;
       i != aDict->end() ;
       ++i) {
    patBoolean known(patFALSE) ;
    if (isRealParam(i->first)) {
      known = patTRUE ;
      patReal r ;
      if (!from_string<patReal>(r,i->second,std::dec)) {
	stringstream str ;
	str << "Error in converting the following string: " << i->second ;
	err = new patErrMiscError(str.str()) ;
	WARNING(err->describe()) ;
	return ;
      }
      setValueReal(i->first,r,err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return ;
      }
    }
    if (isStringParam(i->first)) {
      known = patTRUE ;
      setValueString(i->first,i->second,err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return ;
      }
    }
    if (isIntParam(i->first)) {
      known = patTRUE ;
      long int r ;
      if (!from_string<long int>(r,i->second,std::dec)) {
	stringstream str ;
	str << "Error in converting the following string: " << i->second ;
	err = new patErrMiscError(str.str()) ;
	WARNING(err->describe()) ;
	return ;
      }
      setValueInt(i->first,r,err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return ;
      }
    }
    if (!known) {
      unknown.push_back(i->first) ;
    }
    
  }
  if (unknown.size() > 0) {
    stringstream str ;
    if (unknown.size() == 1) {
      str << "Unknown parameter: " << unknown[0] ;
    }
    else {
      str << "Unknown parameters: " ;
      for (int i = 0 ; i < unknown.size() ; ++i) {
	if (i != 0) {
	  str << ", " ;
	}
	str << unknown[i] ;
      }
    }
    str << ". The list of parameters can be found in the file __parametersUsed.py." ;
    err = new patErrMiscError(str.str()) ;
    WARNING(err->describe()) ;
    return ;
  }

  return ;
}

patString bioParameters::printPythonCode() const {
  stringstream str ;
  for (map<patString,patReal>::const_iterator i = realValues.begin() ;
       i != realValues.end() ;
       ++i) {
    str << "BIOGEME_OBJECT.PARAMETERS['"<<i->first<<"'] = \""<<i->second<<"\"" << endl ;
  }
  for (map<patString,patString>::const_iterator i = stringValues.begin() ;
       i != stringValues.end() ;
       ++i) {
    str << "BIOGEME_OBJECT.PARAMETERS['"<<i->first<<"'] = \""<<i->second<<"\"" << endl ;
  }
  for (map<patString,long int>::const_iterator i = integerValues.begin() ;
       i != integerValues.end() ;
       ++i) {
    str << "BIOGEME_OBJECT.PARAMETERS['"<<i->first<<"'] = \""<<i->second<<"\"" << endl ;
  }
  return patString(str.str()) ;


}

trParameters bioParameters::getTrParameters(patError*& err) const {

  trParameters p ;
  p.eta1 = getValueReal("BTREta1",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.eta2 = getValueReal("BTREta2",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.gamma2 = getValueReal("BTRGamma2",err);
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.beta = getValueReal("BTRIncreaseTRRadius",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.maxIter = getValueInt("BTRMaxIter",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.initQuasiNewtonWithTrueHessian = 
    getValueInt("BTRInitQuasiNewtonWithTrueHessian",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.significantDigits = getValueInt("BTRSignificantDigits",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.usePreconditioner = getValueInt("BTRUsePreconditioner",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.maxTrustRegionRadius = getValueReal("BTRMaxTRRadius",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.typicalF = getValueReal("BTRTypf",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.tolerance = getValueReal("BTRTolerance",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.toleranceSchnabelEskow = getValueReal("TolSchnabelEskow",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.exactHessian = getValueInt("BTRExactHessian",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.cheapHessian = getValueInt("BTRCheapHessian",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.initRadius = getValueReal("BTRInitRadius",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.minRadius = getValueReal("BTRMinTRRadius",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.armijoBeta1 = getValueReal("BTRArmijoBeta1",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.armijoBeta2 = getValueReal("BTRArmijoBeta2",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.stopFileName = getValueString("stopFileName",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.startDraws = getValueInt("BTRStartDraws",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.increaseDraws = getValueInt("BTRIncreaseDraws",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.maxGcpIter = getValueInt("maxGcpIter",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.fractionGradientRequired = getValueReal("TSFractionGradientRequired",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.expTheta = getValueReal("TSExpTheta",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.infeasibleCgIter = getValueInt("BTRUnfeasibleCGIterations",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.quasiNewtonUpdate = getValueInt("BTRQuasiNewtonUpdate",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.kappaUbs = getValueReal("BTRKappaUbs",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.kappaLbs = getValueReal("BTRKappaLbs",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.kappaFrd = getValueReal("BTRKappaFrd",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }
  p.kappaEpp = getValueReal("BTRKappaEpp",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return trParameters() ;
  }

  return p ;

}

