//-*-c++-*------------------------------------------------------------
//
// File name : bioRandomDraws.h
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Thu Jul 30 16:45:35 2009
//
//--------------------------------------------------------------------

#ifndef bioRandomDraws_h
#define bioRandomDraws_h

#include <map>
#include "patError.h"
#include "patString.h"
#include "patVariables.h"

class bioDrawIterator ;
class patUniform ;
class patRandomNumberGenerator ;
class bioSample ;

class bioRandomDraws {

  friend ostream& operator<<(ostream& str, const bioRandomDraws& x) ;

 public:
  typedef enum {
    NORMAL,
    UNIFORM,
    UNIFORMSYM,
    UNDEFINED
  } bioDrawsType ;
  
public:
  static bioRandomDraws* the() ;
  patULong addRandomVariable(patString name, bioDrawsType type, patString index, patError*& err) ;
  void generateDraws(patError*& err) ;
  vector< vector<patReal> >*  getRow(patULong rowId, patError*& err) ;
  patULong getColId(patString name, patError*& err) ;
  bioDrawIterator* createIterator() ;
  patBoolean hasRandomVariables() const ;
  void populateListOfIds(bioSample* aSample, patError*& err) ;
  patULong nbrOfDraws() const ;
 private:
  bioRandomDraws() ;
  // Names of the random variables
  vector<patString> names ;
  // Names of the column in the data file defining the
  // index. Typically, the id of an observation or of an individual.
  vector<patString> indexNames ;
  // Index within a data row of the above index.
  vector<patULong> indexIds ;
  // Type of each random variable (normal or uniform)
  vector<bioDrawsType> types ;
  // For each random variables, list of the values thatthe index can
  // take. For instance, the list of observation ids, or the list of
  // individualids.
  vector<vector<patULong> > listOfIds ;
  // Database contains draws[drawNumber][beta][individualId]
  vector<vector< vector<patReal> > > draws ;
  //  vector<vector< map<patULong,patReal> > > draws ;
  map<patString,patULong> idFromName ;
  // Just for reporting. Translate the type of draw into a string
  vector<patString> typeNames ;
  patUniform* rng ;
  patRandomNumberGenerator* theNormalGenerator ;
  patRandomNumberGenerator* theRectangularGenerator ;
  patRandomNumberGenerator* theZeroOneGenerator ;

};
#endif
